//+------------------------------------------------------------------+
//|                                                 SOM_Net_Data.mqh |
//|                                            Copyright 2015, Urain |
//|                            https://login.mql5.com/ru/users/Urain |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, Urain"
#property link      "https://login.mql5.com/ru/users/Urain"
//+------------------------------------------------------------------+
//| #include <SOM_Net_Data.mqh>                                      |
//+------------------------------------------------------------------+
#include "SOM_Net_Base.mqh"
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CSOM_Net_Data : public CSOM_Net_Base
  {
protected:
   //---    
   int               m_total_patterns_sets;
   int               m_size_patterns_sets_array;
   //---    
   double            m_patterns_sets_array[];
   string            m_patterns_titles[];
   //---     CSV-
   void              ReadCSVData(int handle);
   //---    CSV-
   void              ParseString(string value,string delimiter,string  &mystrings[]);
public:
                     CSOM_Net_Data(void){};
                    ~CSOM_Net_Data(void){};
   //---        
   bool              LoadPatternDataFromFile(string filename);
   //---      
   void              AddVectorToPatternsSet(double &vector[],string title);
   //---       -    
   bool              GetPatterns(int ind_pattern,double &vector[]);
   //---        - 
   string            GetTitlesPatterns(int ind_pattern){return(m_patterns_titles[ind_pattern]);};
   //---     
   int               GetTotalsPatterns(){return(m_total_patterns_sets);};
  };
//+------------------------------------------------------------------+
//|       filename          |
//+------------------------------------------------------------------+
bool CSOM_Net_Data::LoadPatternDataFromFile(string filename)
  {
//---
   ResetLastError();
   int filehandle=FileOpen(filename,FILE_READ|FILE_ANSI);
   if(filehandle!=INVALID_HANDLE)
     {
      Print("FileOpen "+filename+" OK");
      m_total_patterns_sets=0;
      ReadCSVData(filehandle);
      FileClose(filehandle);
      return(true);
     }
   else
     {
      Print("   ",filename," : ",GetLastError());
      return(false);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CSOM_Net_Data::ReadCSVData(int handle)
  {
   string stringsarr[];
   string s;
//---
   int strings_total=0;
   while(!FileIsEnding(handle))
     {
      if(IsStopped())break;
      s=FileReadString(handle);
      if(StringLen(s)>0)
        {
         strings_total++;
         //---
         if(strings_total>1)
           {
            ParseString(s,";",stringsarr);
            double datavector[];
            int dimension=ArraySize(stringsarr);
            ArrayResize(datavector,dimension);
            //---
            for(int i=0; i<dimension; i++)
               datavector[i]=StringToDouble(stringsarr[i]);
            //---     
            AddVectorToPatternsSet(datavector,"#"+(string)strings_total);
           }
         else
           {
            ParseString(s,";",stringsarr);
            Print("HEADER: "+IntegerToString(ArraySize(stringsarr)));
            //---  ,  10000 
            //---  CellsX*CellsY ,   ImageXSize x ImageYSize
            m_dimension=ArraySize(stringsarr);
            //---   
            int total_stringsarr=ArrayResize(m_som_titles,ArraySize(stringsarr));
            for(int i=0; i<total_stringsarr; i++)
               m_som_titles[i]=stringsarr[i];
           }
        }
     }
   m_size_patterns_sets_array=ArraySize(m_patterns_sets_array);
   Print("Dimension=",m_dimension);
  };
//+------------------------------------------------------------------+
//|    ,  delimiter                |
//+------------------------------------------------------------------+
void CSOM_Net_Data::ParseString(string value,string delimiter,string  &mystrings[])
  {
   int indx;
   int dlen=StringLen(delimiter);
   string txt=value+delimiter;
   int strind=0;
   while(StringLen(txt)>0)
     {
      strind++;
      indx=StringFind(txt,delimiter);
      string tempstr=StringSubstr(txt,0,indx);
      txt=StringSubstr(txt,indx+dlen,-1);
      ArrayResize(mystrings,strind,50);
      mystrings[strind-1]=tempstr;
     }
  };
//+------------------------------------------------------------------+
//|                                |
//+------------------------------------------------------------------+
void CSOM_Net_Data::AddVectorToPatternsSet(double &vector[],string title)
  {
   m_total_patterns_sets++;
   ArrayResize(m_patterns_sets_array,m_dimension*m_total_patterns_sets);
   ArrayResize(m_patterns_titles,m_total_patterns_sets);
   m_patterns_titles[m_total_patterns_sets-1]=title;
   for(int i=0; i<m_dimension; i++)
     {
      m_patterns_sets_array[m_dimension*(m_total_patterns_sets-1)+i]=vector[i];
     }
  };
//+------------------------------------------------------------------+
//|                     |
//+------------------------------------------------------------------+
bool CSOM_Net_Data::GetPatterns(int ind_pattern,double &vector[])
  {
   ArrayResize(vector,m_dimension);
   if(m_dimension*(ind_pattern+1)<=m_size_patterns_sets_array)
     {
      for(int k=0; k<m_dimension && !IsStopped(); k++)
        {
         vector[k]=m_patterns_sets_array[m_dimension*(ind_pattern)+k];
        }
      return(1);
     }
   else return(0);
  };
//+------------------------------------------------------------------+
